@@ -5,20 +5,23 @@
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 from . import Connector
-from ..util import asbool
+from .. import util
+
 
 import sys
 import re
-import urllib
 
 
 class PyODBCConnector(Connector):
     driver = 'pyodbc'
 
     supports_sane_multi_rowcount = False
-    # PyODBC unicode is broken on UCS-4 builds
-    supports_unicode = sys.maxunicode == 65535
-    supports_unicode_statements = supports_unicode
+
+    if util.py2k:
+        # PyODBC unicode is broken on UCS-4 builds
+        supports_unicode = sys.maxunicode == 65535
+        supports_unicode_statements = supports_unicode
+
     supports_native_decimal = True
     default_paramstyle = 'named'
 
@@ -56,10 +59,10 @@ class PyODBCConnector(Connector):
         connect_args = {}
         for param in ('ansi', 'unicode_results', 'autocommit'):
             if param in keys:
-                connect_args[param] = asbool(keys.pop(param))
+                connect_args[param] = util.asbool(keys.pop(param))
 
         if 'odbc_connect' in keys:
-            connectors = [urllib.unquote_plus(keys.pop('odbc_connect'))]
+            connectors = [util.unquote_plus(keys.pop('odbc_connect'))]
         else:
             dsn_connection = 'dsn' in keys or \
                             ('host' in keys and 'database' not in keys)
@@ -91,7 +94,7 @@ class PyODBCConnector(Connector):
                 connectors.append("AutoTranslate=%s" %
                                     keys.pop("odbc_autotranslate"))
 
-            connectors.extend(['%s=%s' % (k, v) for k, v in keys.iteritems()])
+            connectors.extend(['%s=%s' % (k, v) for k, v in keys.items()])
         return [[";".join(connectors)], connect_args]
 
     def is_disconnect(self, e, connection, cursor):
@@ -121,23 +124,23 @@ class PyODBCConnector(Connector):
             self.freetds_driver_version = dbapi_con.getinfo(
                 pyodbc.SQL_DRIVER_VER)
 
-        # the "Py2K only" part here is theoretical.
-        # have not tried pyodbc + python3.1 yet.
-        # Py2K
         self.supports_unicode_statements = (
-            not self.freetds and not self.easysoft)
+            not util.py2k or
+            (not self.freetds and not self.easysoft)
+        )
+
         if self._user_supports_unicode_binds is not None:
             self.supports_unicode_binds = self._user_supports_unicode_binds
-        else:
+        elif util.py2k:
             self.supports_unicode_binds = (
                 not self.freetds or self.freetds_driver_version >= '0.91'
             ) and not self.easysoft
-        # end Py2K
+        else:
+            self.supports_unicode_binds = True
 
         # run other initialization which asks for user name, etc.
         super(PyODBCConnector, self).initialize(connection)
 
-
     def _dbapi_version(self):
         if not self.dbapi:
             return ()
