@@ -7,7 +7,7 @@
 from .. import event, util
 from .interfaces import EXT_CONTINUE
 
-
+@util.langhelpers.dependency_for("sqlalchemy.orm.interfaces")
 class MapperExtension(object):
     """Base implementation for :class:`.Mapper` event hooks.
 
@@ -127,7 +127,7 @@ class MapperExtension(object):
         return EXT_CONTINUE
 
     def init_instance(self, mapper, class_, oldinit, instance, args, kwargs):
-        """Receive an instance when its constructor is called.
+        """Receive an instance when it's constructor is called.
 
         This method is only called during a userland construction of
         an object.  It is not called when an object is loaded from the
@@ -140,7 +140,7 @@ class MapperExtension(object):
         return EXT_CONTINUE
 
     def init_failed(self, mapper, class_, oldinit, instance, args, kwargs):
-        """Receive an instance when its constructor has been called,
+        """Receive an instance when it's constructor has been called,
         and raised an exception.
 
         This method is only called during a userland construction of
@@ -374,6 +374,7 @@ class MapperExtension(object):
         return EXT_CONTINUE
 
 
+@util.langhelpers.dependency_for("sqlalchemy.orm.interfaces")
 class SessionExtension(object):
 
     """Base implementation for :class:`.Session` event hooks.
@@ -494,6 +495,7 @@ class SessionExtension(object):
         """
 
 
+@util.langhelpers.dependency_for("sqlalchemy.orm.interfaces")
 class AttributeExtension(object):
     """Base implementation for :class:`.AttributeImpl` event hooks, events
     that fire upon attribute mutations in user code.
