@@ -1,4 +1,9 @@
-from __future__ import with_statement
+# testing/exclusions.py
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
 
 import operator
 from nose import SkipTest
@@ -28,10 +33,10 @@ class skip_if(object):
     def fail_if(self, name='block'):
         try:
             yield
-        except Exception, ex:
+        except Exception as ex:
             if self.predicate():
-                print ("%s failed as expected (%s): %s " % (
-                    name, self.predicate, str(ex)))
+                print(("%s failed as expected (%s): %s " % (
+                    name, self.predicate, str(ex))))
             else:
                 raise
         else:
@@ -66,9 +71,6 @@ class skip_if(object):
         self._fails_on = skip_if(other, reason)
         return self
 
-    def fails_on_everything_except(self, *dbs):
-        self._fails_on = skip_if(fails_on_everything_except(*dbs))
-        return self
 
 class fails_if(skip_if):
     def __call__(self, fn):
@@ -100,7 +102,7 @@ class Predicate(object):
             return OrPredicate([cls.as_predicate(pred) for pred in predicate])
         elif isinstance(predicate, tuple):
             return SpecPredicate(*predicate)
-        elif isinstance(predicate, basestring):
+        elif isinstance(predicate, util.string_types):
             tokens = predicate.split(" ", 2)
             op = spec = None
             db = tokens.pop(0)
