@@ -1,10 +1,16 @@
+# testing/requirements.py
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
 """Global database feature support policy.
 
 Provides decorators to mark tests requiring specific feature support from the
 target database.
 
 External dialect test suites should subclass SuiteRequirements
-to provide specific inclusion/exclusions.
+to provide specific inclusion/exlusions.
 
 """
 
@@ -46,12 +52,6 @@ class SuiteRequirements(Requirements):
 
         return exclusions.open()
 
-    @property
-    def non_updating_cascade(self):
-        """target database must *not* support ON UPDATE..CASCADE behavior in
-        foreign keys."""
-        return exclusions.closed()
-
     @property
     def deferrable_fks(self):
         return exclusions.closed()
@@ -315,6 +315,15 @@ class SuiteRequirements(Requirements):
         """Target driver must support some degree of non-ascii symbol names."""
         return exclusions.closed()
 
+    @property
+    def datetime_literals(self):
+        """target dialect supports rendering of a date, time, or datetime as a
+        literal string, e.g. via the TypeEngine.literal_processor() method.
+
+        """
+
+        return exclusions.closed()
+
     @property
     def datetime(self):
         """target dialect supports representation of Python
@@ -364,6 +373,30 @@ class SuiteRequirements(Requirements):
 
         return exclusions.open()
 
+    @property
+    def binary_comparisons(self):
+        """target database/driver can allow BLOB/BINARY fields to be compared
+        against a bound parameter value.
+        """
+
+        return exclusions.open()
+
+    @property
+    def binary_literals(self):
+        """target backend supports simple binary literals, e.g. an
+        expression like::
+
+            SELECT CAST('foo' AS BINARY)
+
+        Where ``BINARY`` is the type emitted from :class:`.LargeBinary`,
+        e.g. it could be ``BLOB`` or similar.
+
+        Basically fails on Oracle.
+
+        """
+
+        return exclusions.open()
+
     @property
     def precision_numerics_general(self):
         """target backend has general support for moderately high-precision
@@ -398,6 +431,14 @@ class SuiteRequirements(Requirements):
 
         return exclusions.closed()
 
+    @property
+    def precision_generic_float_type(self):
+        """target backend will return native floating point numbers with at
+        least seven decimal places when using the generic Float type.
+
+        """
+        return exclusions.open()
+
     @property
     def floats_to_four_decimals(self):
         """target backend can return a floating-point number with four
