@@ -1,14 +1,23 @@
+#!coding: utf-8
+
 """SQLite-specific tests."""
 
-from test.lib.testing import eq_, assert_raises, \
+from sqlalchemy.testing import eq_, assert_raises, \
     assert_raises_message
 import datetime
-from sqlalchemy import *
-from sqlalchemy import exc, sql, schema, pool, types as sqltypes
+from sqlalchemy import Table, String, select, Text, CHAR, bindparam, Column,\
+    Unicode, Date, MetaData, UnicodeText, Time, Integer, TIMESTAMP, \
+    Boolean, func, NUMERIC, DateTime, extract, ForeignKey, text, Numeric,\
+    DefaultClause, and_, DECIMAL, TypeDecorator, create_engine, Float, \
+    INTEGER, UniqueConstraint, DATETIME, DATE, TIME, BOOLEAN, BIGINT
+from sqlalchemy.util import u, ue
+from sqlalchemy import exc, sql, schema, pool, types as sqltypes, util
 from sqlalchemy.dialects.sqlite import base as sqlite, \
     pysqlite as pysqlite_dialect
 from sqlalchemy.engine.url import make_url
-from test.lib import *
+from sqlalchemy.testing import fixtures, AssertsCompiledSQL, \
+    AssertsExecutionResults, engines
+from sqlalchemy import testing
 import os
 from sqlalchemy.schema import CreateTable
 
@@ -63,15 +72,6 @@ class TestTypes(fixtures.TestBase, AssertsExecutionResults):
                 ).scalar()
             )
 
-    def test_time_microseconds(self):
-        dt = datetime.datetime(2008, 6, 27, 12, 0, 0, 125, )
-        eq_(str(dt), '2008-06-27 12:00:00.000125')
-        sldt = sqlite.DATETIME()
-        bp = sldt.bind_processor(None)
-        eq_(bp(dt), '2008-06-27 12:00:00.000125')
-        rp = sldt.result_processor(None, None)
-        eq_(rp(bp(dt)), dt)
-
     def test_native_datetime(self):
         dbapi = testing.db.dialect.dbapi
         connect_args = {'detect_types': dbapi.PARSE_DECLTYPES \
@@ -91,11 +91,57 @@ class TestTypes(fixtures.TestBase, AssertsExecutionResults):
             eq_(row, (1, datetime.date(2010, 5, 10),
             datetime.datetime( 2010, 5, 10, 12, 15, 25, )))
             r = engine.execute(func.current_date()).scalar()
-            assert isinstance(r, basestring)
+            assert isinstance(r, util.string_types)
         finally:
             t.drop(engine)
             engine.dispose()
 
+    @testing.provide_metadata
+    def test_custom_datetime(self):
+        sqlite_date = sqlite.DATETIME(
+                # 2004-05-21T00:00:00
+                storage_format="%(year)04d-%(month)02d-%(day)02d"
+                    "T%(hour)02d:%(minute)02d:%(second)02d",
+                regexp=r"(\d+)-(\d+)-(\d+)T(\d+):(\d+):(\d+)",
+            )
+        t = Table('t', self.metadata, Column('d', sqlite_date))
+        self.metadata.create_all(testing.db)
+        testing.db.execute(t.insert().
+                        values(d=datetime.datetime(2010, 10, 15, 12, 37, 0)))
+        testing.db.execute("insert into t (d) values ('2004-05-21T00:00:00')")
+        eq_(
+            testing.db.execute("select * from t order by d").fetchall(),
+            [(u'2004-05-21T00:00:00',), (u'2010-10-15T12:37:00',)]
+        )
+        eq_(
+            testing.db.execute(select([t.c.d]).order_by(t.c.d)).fetchall(),
+            [(datetime.datetime(2004, 5, 21, 0, 0),),
+            (datetime.datetime(2010, 10, 15, 12, 37),)]
+        )
+
+    @testing.provide_metadata
+    def test_custom_date(self):
+        sqlite_date = sqlite.DATE(
+                # 2004-05-21T00:00:00
+                storage_format="%(year)04d|%(month)02d|%(day)02d",
+                regexp=r"(\d+)\|(\d+)\|(\d+)",
+            )
+        t = Table('t', self.metadata, Column('d', sqlite_date))
+        self.metadata.create_all(testing.db)
+        testing.db.execute(t.insert().
+                        values(d=datetime.date(2010, 10, 15)))
+        testing.db.execute("insert into t (d) values ('2004|05|21')")
+        eq_(
+            testing.db.execute("select * from t order by d").fetchall(),
+            [(u'2004|05|21',), (u'2010|10|15',)]
+        )
+        eq_(
+            testing.db.execute(select([t.c.d]).order_by(t.c.d)).fetchall(),
+            [(datetime.date(2004, 5, 21),),
+            (datetime.date(2010, 10, 15),)]
+        )
+
+
     def test_no_convert_unicode(self):
         """test no utf-8 encoding occurs"""
 
@@ -111,8 +157,8 @@ class TestTypes(fixtures.TestBase, AssertsExecutionResults):
             UnicodeText(),
             ):
             bindproc = t.dialect_impl(dialect).bind_processor(dialect)
-            assert not bindproc or isinstance(bindproc(u'some string'),
-                    unicode)
+            assert not bindproc or \
+                isinstance(bindproc(util.u('some string')), util.text_type)
 
     @testing.provide_metadata
     def test_type_reflection(self):
@@ -137,6 +183,8 @@ class TestTypes(fixtures.TestBase, AssertsExecutionResults):
             (Numeric(10, 2), NUMERIC(10, 2)),
             (DECIMAL, DECIMAL()),
             (DECIMAL(10, 2), DECIMAL(10, 2)),
+            (INTEGER, INTEGER()),
+            (BIGINT, BIGINT()),
             (Float, Float()),
             (NUMERIC(), ),
             (TIMESTAMP, TIMESTAMP()),
@@ -183,6 +231,99 @@ class TestTypes(fixtures.TestBase, AssertsExecutionResults):
         assert isinstance(t2.c.y.type, sqltypes.NullType)
 
 
+class DateTimeTest(fixtures.TestBase, AssertsCompiledSQL):
+
+    def test_time_microseconds(self):
+        dt = datetime.datetime(2008, 6, 27, 12, 0, 0, 125, )
+        eq_(str(dt), '2008-06-27 12:00:00.000125')
+        sldt = sqlite.DATETIME()
+        bp = sldt.bind_processor(None)
+        eq_(bp(dt), '2008-06-27 12:00:00.000125')
+        rp = sldt.result_processor(None, None)
+        eq_(rp(bp(dt)), dt)
+
+    def test_truncate_microseconds(self):
+        dt = datetime.datetime(2008, 6, 27, 12, 0, 0, 125)
+        dt_out = datetime.datetime(2008, 6, 27, 12, 0, 0)
+        eq_(str(dt), '2008-06-27 12:00:00.000125')
+        sldt = sqlite.DATETIME(truncate_microseconds=True)
+        bp = sldt.bind_processor(None)
+        eq_(bp(dt), '2008-06-27 12:00:00')
+        rp = sldt.result_processor(None, None)
+        eq_(rp(bp(dt)), dt_out)
+
+    def test_custom_format_compact(self):
+        dt = datetime.datetime(2008, 6, 27, 12, 0, 0, 125)
+        eq_(str(dt), '2008-06-27 12:00:00.000125')
+        sldt = sqlite.DATETIME(
+            storage_format=(
+                "%(year)04d%(month)02d%(day)02d"
+                "%(hour)02d%(minute)02d%(second)02d%(microsecond)06d"
+            ),
+            regexp="(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})(\d{6})",
+        )
+        bp = sldt.bind_processor(None)
+        eq_(bp(dt), '20080627120000000125')
+        rp = sldt.result_processor(None, None)
+        eq_(rp(bp(dt)), dt)
+
+class DateTest(fixtures.TestBase, AssertsCompiledSQL):
+
+    def test_default(self):
+        dt = datetime.date(2008, 6, 27)
+        eq_(str(dt), '2008-06-27')
+        sldt = sqlite.DATE()
+        bp = sldt.bind_processor(None)
+        eq_(bp(dt), '2008-06-27')
+        rp = sldt.result_processor(None, None)
+        eq_(rp(bp(dt)), dt)
+
+    def test_custom_format(self):
+        dt = datetime.date(2008, 6, 27)
+        eq_(str(dt), '2008-06-27')
+        sldt = sqlite.DATE(
+            storage_format="%(month)02d/%(day)02d/%(year)04d",
+            regexp="(?P<month>\d+)/(?P<day>\d+)/(?P<year>\d+)",
+        )
+        bp = sldt.bind_processor(None)
+        eq_(bp(dt), '06/27/2008')
+        rp = sldt.result_processor(None, None)
+        eq_(rp(bp(dt)), dt)
+
+class TimeTest(fixtures.TestBase, AssertsCompiledSQL):
+
+    def test_default(self):
+        dt = datetime.date(2008, 6, 27)
+        eq_(str(dt), '2008-06-27')
+        sldt = sqlite.DATE()
+        bp = sldt.bind_processor(None)
+        eq_(bp(dt), '2008-06-27')
+        rp = sldt.result_processor(None, None)
+        eq_(rp(bp(dt)), dt)
+
+    def test_truncate_microseconds(self):
+        dt = datetime.time(12, 0, 0, 125)
+        dt_out = datetime.time(12, 0, 0)
+        eq_(str(dt), '12:00:00.000125')
+        sldt = sqlite.TIME(truncate_microseconds=True)
+        bp = sldt.bind_processor(None)
+        eq_(bp(dt), '12:00:00')
+        rp = sldt.result_processor(None, None)
+        eq_(rp(bp(dt)), dt_out)
+
+    def test_custom_format(self):
+        dt = datetime.date(2008, 6, 27)
+        eq_(str(dt), '2008-06-27')
+        sldt = sqlite.DATE(
+            storage_format="%(year)04d%(month)02d%(day)02d",
+            regexp="(\d{4})(\d{2})(\d{2})",
+        )
+        bp = sldt.bind_processor(None)
+        eq_(bp(dt), '20080627')
+        rp = sldt.result_processor(None, None)
+        eq_(rp(bp(dt)), dt)
+
+
 class DefaultsTest(fixtures.TestBase, AssertsCompiledSQL):
 
     __only_on__ = 'sqlite'
@@ -398,6 +539,20 @@ class DialectTest(fixtures.TestBase, AssertsExecutionResults):
                     'constrained_columns': ['tid']
                 }])
 
+    @testing.provide_metadata
+    def test_description_encoding(self):
+        # amazingly, pysqlite seems to still deliver cursor.description
+        # as encoded bytes in py2k
+
+        t = Table('x', self.metadata,
+                Column(u('méil'), Integer, primary_key=True),
+                Column(ue('\u6e2c\u8a66'), Integer),
+            )
+        self.metadata.create_all(testing.db)
+
+        result = testing.db.execute(t.select())
+        assert u('méil') in result.keys()
+        assert ue('\u6e2c\u8a66') in result.keys()
 
     def test_attached_as_schema(self):
         cx = testing.db.connect()
@@ -479,7 +634,7 @@ class DialectTest(fixtures.TestBase, AssertsExecutionResults):
             eq_(inspector.get_indexes('foo'), [])
             eq_(inspector.get_indexes('foo',
                 include_auto_indexes=True), [{'unique': 1, 'name'
-                : u'sqlite_autoindex_foo_1', 'column_names': [u'bar']}])
+                : 'sqlite_autoindex_foo_1', 'column_names': ['bar']}])
         finally:
             meta.drop_all()
 
