@@ -1,15 +1,16 @@
-from test.lib.testing import eq_, assert_raises, \
+from sqlalchemy.testing import eq_, assert_raises, \
     assert_raises_message, ne_
 import sys
 import time
 import threading
-from test.lib.engines import testing_engine
+from sqlalchemy import event
+from sqlalchemy.testing.engines import testing_engine
 from sqlalchemy import create_engine, MetaData, INT, VARCHAR, Sequence, \
     select, Integer, String, func, text, exc
-from test.lib.schema import Table
-from test.lib.schema import Column
-from test.lib import testing
-from test.lib import fixtures
+from sqlalchemy.testing.schema import Table
+from sqlalchemy.testing.schema import Column
+from sqlalchemy import testing
+from sqlalchemy.testing import fixtures
 
 
 users, metadata = None, None
@@ -73,8 +74,8 @@ class TransactionTest(fixtures.TestBase):
             connection.execute(users.insert(), user_id=1, user_name='user3')
             transaction.commit()
             assert False
-        except Exception , e:
-            print "Exception: ", e
+        except Exception as e:
+            print("Exception: ", e)
             transaction.rollback()
 
         result = connection.execute("select * from query_users")
@@ -120,10 +121,10 @@ class TransactionTest(fixtures.TestBase):
                     trans2.rollback()
                     raise
                 transaction.rollback()
-            except Exception, e:
+            except Exception as e:
                 transaction.rollback()
                 raise
-        except Exception, e:
+        except Exception as e:
             try:
                 assert str(e) == 'uh oh'  # and not "This transaction is
                                           # inactive"
@@ -159,6 +160,28 @@ class TransactionTest(fixtures.TestBase):
         assert len(result.fetchall()) == 0
         connection.close()
 
+    def test_with_interface(self):
+        connection = testing.db.connect()
+        trans = connection.begin()
+        connection.execute(users.insert(), user_id=1, user_name='user1')
+        connection.execute(users.insert(), user_id=2, user_name='user2')
+        try:
+            connection.execute(users.insert(), user_id=2, user_name='user2.5')
+        except Exception as e:
+            trans.__exit__(*sys.exc_info())
+
+        assert not trans.is_active
+        self.assert_(connection.scalar('select count(*) from '
+                     'query_users') == 0)
+
+        trans = connection.begin()
+        connection.execute(users.insert(), user_id=1, user_name='user1')
+        trans.__exit__(None, None, None)
+        assert not trans.is_active
+        self.assert_(connection.scalar('select count(*) from '
+                     'query_users') == 1)
+        connection.close()
+
     def test_close(self):
         connection = testing.db.connect()
         transaction = connection.begin()
@@ -276,6 +299,7 @@ class TransactionTest(fixtures.TestBase):
     # PG emergency shutdown:
     # select * from pg_prepared_xacts
     # ROLLBACK PREPARED '<xid>'
+    @testing.crashes('mysql', 'Crashing on 5.5, not worth it')
     @testing.requires.skip_mysql_on_windows
     @testing.requires.two_phase_transactions
     @testing.requires.savepoints
@@ -318,7 +342,8 @@ class TransactionTest(fixtures.TestBase):
         transaction = connection.begin_twophase()
         connection.execute(users.insert(), user_id=1, user_name='user1')
         transaction.prepare()
-        connection.close()
+        connection.invalidate()
+
         connection2 = testing.db.connect()
         eq_(connection2.execute(select([users.c.user_id]).
             order_by(users.c.user_id)).fetchall(),
@@ -355,6 +380,138 @@ class TransactionTest(fixtures.TestBase):
         eq_(result.fetchall(), [('user1', ), ('user4', )])
         conn.close()
 
+    @testing.requires.two_phase_transactions
+    def test_reset_rollback_two_phase_no_rollback(self):
+        # test [ticket:2907], essentially that the
+        # TwoPhaseTransaction is given the job of "reset on return"
+        # so that picky backends like MySQL correctly clear out
+        # their state when a connection is closed without handling
+        # the transaction explicitly.
+
+        eng = testing_engine()
+
+        # MySQL raises if you call straight rollback() on
+        # a connection with an XID present
+        @event.listens_for(eng, "invalidate")
+        def conn_invalidated(dbapi_con, con_record, exception):
+            dbapi_con.close()
+            raise exception
+
+        with eng.connect() as conn:
+            rec = conn.connection._connection_record
+            raw_dbapi_con = rec.connection
+            xa = conn.begin_twophase()
+            conn.execute(users.insert(), user_id=1, user_name='user1')
+
+        assert rec.connection is raw_dbapi_con
+
+        with eng.connect() as conn:
+            result = \
+                conn.execute(select([users.c.user_name]).
+                    order_by(users.c.user_id))
+            eq_(result.fetchall(), [])
+
+class ResetAgentTest(fixtures.TestBase):
+    def test_begin_close(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            assert connection.connection._reset_agent is trans
+        assert not trans.is_active
+
+    def test_begin_rollback(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            assert connection.connection._reset_agent is trans
+            trans.rollback()
+            assert connection.connection._reset_agent is None
+
+    def test_begin_commit(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            assert connection.connection._reset_agent is trans
+            trans.commit()
+            assert connection.connection._reset_agent is None
+
+    @testing.requires.savepoints
+    def test_begin_nested_close(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin_nested()
+            assert connection.connection._reset_agent is trans
+        assert not trans.is_active
+
+    @testing.requires.savepoints
+    def test_begin_begin_nested_close(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            trans2 = connection.begin_nested()
+            assert connection.connection._reset_agent is trans
+        assert trans2.is_active  # was never closed
+        assert not trans.is_active
+
+    @testing.requires.savepoints
+    def test_begin_begin_nested_rollback_commit(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            trans2 = connection.begin_nested()
+            assert connection.connection._reset_agent is trans
+            trans2.rollback()
+            assert connection.connection._reset_agent is trans
+            trans.commit()
+            assert connection.connection._reset_agent is None
+
+    @testing.requires.savepoints
+    def test_begin_begin_nested_rollback_rollback(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            trans2 = connection.begin_nested()
+            assert connection.connection._reset_agent is trans
+            trans2.rollback()
+            assert connection.connection._reset_agent is trans
+            trans.rollback()
+            assert connection.connection._reset_agent is None
+
+    def test_begin_begin_rollback_rollback(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            trans2 = connection.begin()
+            assert connection.connection._reset_agent is trans
+            trans2.rollback()
+            assert connection.connection._reset_agent is None
+            trans.rollback()
+            assert connection.connection._reset_agent is None
+
+    def test_begin_begin_commit_commit(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            trans2 = connection.begin()
+            assert connection.connection._reset_agent is trans
+            trans2.commit()
+            assert connection.connection._reset_agent is trans
+            trans.commit()
+            assert connection.connection._reset_agent is None
+
+    @testing.requires.two_phase_transactions
+    def test_reset_via_agent_begin_twophase(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin_twophase()
+            assert connection.connection._reset_agent is trans
+
+    @testing.requires.two_phase_transactions
+    def test_reset_via_agent_begin_twophase_commit(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin_twophase()
+            assert connection.connection._reset_agent is trans
+            trans.commit()
+            assert connection.connection._reset_agent is None
+
+    @testing.requires.two_phase_transactions
+    def test_reset_via_agent_begin_twophase_rollback(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin_twophase()
+            assert connection.connection._reset_agent is trans
+            trans.rollback()
+            assert connection.connection._reset_agent is None
+
 class AutoRollbackTest(fixtures.TestBase):
 
     @classmethod
@@ -480,7 +637,7 @@ class ExplicitAutoCommitTest(fixtures.TestBase):
         conn2.close()
 
     @testing.uses_deprecated(r'autocommit on select\(\) is deprecated',
-                             r'autocommit\(\) is deprecated')
+                             r'``autocommit\(\)`` is deprecated')
     def test_explicit_compiled_deprecated(self):
         conn1 = testing.db.connect()
         conn2 = testing.db.connect()
@@ -985,17 +1142,11 @@ class ForUpdateTest(fixtures.TestBase):
     def teardown_class(cls):
         counters.drop(testing.db)
 
-    def increment(
-        self,
-        count,
-        errors,
-        update_style=True,
-        delay=0.005,
-        ):
+    def increment(self, count, errors, update_style=True, delay=0.005):
         con = testing.db.connect()
         sel = counters.select(for_update=update_style,
                               whereclause=counters.c.counter_id == 1)
-        for i in xrange(count):
+        for i in range(count):
             trans = con.begin()
             try:
                 existing = con.execute(sel).first()
@@ -1009,7 +1160,7 @@ class ForUpdateTest(fixtures.TestBase):
                     raise AssertionError('Got %s post-update, expected '
                             '%s' % (readback['counter_value'], incr))
                 trans.commit()
-            except Exception, e:
+            except Exception as e:
                 trans.rollback()
                 errors.append(e)
                 break
@@ -1018,7 +1169,6 @@ class ForUpdateTest(fixtures.TestBase):
     @testing.crashes('mssql', 'FIXME: unknown')
     @testing.crashes('firebird', 'FIXME: unknown')
     @testing.crashes('sybase', 'FIXME: unknown')
-    @testing.crashes('access', 'FIXME: unknown')
     @testing.requires.independent_connections
     def test_queued_update(self):
         """Test SELECT FOR UPDATE with concurrent modifications.
@@ -1033,7 +1183,7 @@ class ForUpdateTest(fixtures.TestBase):
         db.execute(counters.insert(), counter_id=1, counter_value=0)
         iterations, thread_count = 10, 5
         threads, errors = [], []
-        for i in xrange(thread_count):
+        for i in range(thread_count):
             thrd = threading.Thread(target=self.increment,
                                     args=(iterations, ),
                                     kwargs={'errors': errors,
@@ -1042,49 +1192,38 @@ class ForUpdateTest(fixtures.TestBase):
             threads.append(thrd)
         for thrd in threads:
             thrd.join()
-        for e in errors:
-            sys.stdout.write('Failure: %s\n' % e)
-        self.assert_(len(errors) == 0)
+        assert not errors
         sel = counters.select(whereclause=counters.c.counter_id == 1)
         final = db.execute(sel).first()
-        self.assert_(final['counter_value'] == iterations
-                     * thread_count)
+        eq_(final['counter_value'], iterations * thread_count)
+
+    def overlap(self, ids, errors, update_style):
 
-    def overlap(
-        self,
-        ids,
-        errors,
-        update_style,
-        ):
         sel = counters.select(for_update=update_style,
                               whereclause=counters.c.counter_id.in_(ids))
         con = testing.db.connect()
         trans = con.begin()
         try:
             rows = con.execute(sel).fetchall()
-            time.sleep(0.25)
+            time.sleep(0.50)
             trans.commit()
-        except Exception, e:
+        except Exception as e:
             trans.rollback()
             errors.append(e)
         con.close()
 
-    def _threaded_overlap(
-        self,
-        thread_count,
-        groups,
-        update_style=True,
-        pool=5,
-        ):
+    def _threaded_overlap(self, thread_count, groups, update_style=True, pool=5):
         db = testing.db
         for cid in range(pool - 1):
             db.execute(counters.insert(), counter_id=cid + 1,
                        counter_value=0)
         errors, threads = [], []
-        for i in xrange(thread_count):
+        for i in range(thread_count):
             thrd = threading.Thread(target=self.overlap,
                                     args=(groups.pop(0), errors,
                                     update_style))
+            time.sleep(0.20)  # give the previous thread a chance to start
+                              # to ensure it gets a lock
             thrd.start()
             threads.append(thrd)
         for thrd in threads:
@@ -1094,28 +1233,24 @@ class ForUpdateTest(fixtures.TestBase):
     @testing.crashes('mssql', 'FIXME: unknown')
     @testing.crashes('firebird', 'FIXME: unknown')
     @testing.crashes('sybase', 'FIXME: unknown')
-    @testing.crashes('access', 'FIXME: unknown')
     @testing.requires.independent_connections
     def test_queued_select(self):
         """Simple SELECT FOR UPDATE conflict test"""
 
         errors = self._threaded_overlap(2, [(1, 2, 3), (3, 4, 5)])
-        for e in errors:
-            sys.stderr.write('Failure: %s\n' % e)
-        self.assert_(len(errors) == 0)
+        assert not errors
 
     @testing.crashes('mssql', 'FIXME: unknown')
     @testing.fails_on('mysql', 'No support for NOWAIT')
     @testing.crashes('firebird', 'FIXME: unknown')
     @testing.crashes('sybase', 'FIXME: unknown')
-    @testing.crashes('access', 'FIXME: unknown')
     @testing.requires.independent_connections
     def test_nowait_select(self):
         """Simple SELECT FOR UPDATE NOWAIT conflict test"""
 
         errors = self._threaded_overlap(2, [(1, 2, 3), (3, 4, 5)],
                 update_style='nowait')
-        self.assert_(len(errors) != 0)
+        assert errors
 
 class IsolationLevelTest(fixtures.TestBase):
     __requires__ = ('isolation_level', 'ad_hoc_engines')
@@ -1125,6 +1260,8 @@ class IsolationLevelTest(fixtures.TestBase):
             return 'SERIALIZABLE'
         elif testing.against('postgresql'):
             return 'READ COMMITTED'
+        elif testing.against('mysql'):
+            return "REPEATABLE READ"
         else:
             assert False, "default isolation level not known"
 
@@ -1133,20 +1270,24 @@ class IsolationLevelTest(fixtures.TestBase):
             return 'READ UNCOMMITTED'
         elif testing.against('postgresql'):
             return 'SERIALIZABLE'
+        elif testing.against('mysql'):
+            return "SERIALIZABLE"
         else:
             assert False, "non default isolation level not known"
 
     def test_engine_param_stays(self):
 
         eng = testing_engine()
-        isolation_level = eng.dialect.get_isolation_level(eng.connect().connection)
+        isolation_level = eng.dialect.get_isolation_level(
+                                eng.connect().connection)
         level = self._non_default_isolation_level()
 
         ne_(isolation_level, level)
 
         eng = testing_engine(options=dict(isolation_level=level))
         eq_(
-            eng.dialect.get_isolation_level(eng.connect().connection),
+            eng.dialect.get_isolation_level(
+                                eng.connect().connection),
             level
         )
 
@@ -1167,62 +1308,91 @@ class IsolationLevelTest(fixtures.TestBase):
 
     def test_default_level(self):
         eng = testing_engine(options=dict())
-        isolation_level = eng.dialect.get_isolation_level(eng.connect().connection)
+        isolation_level = eng.dialect.get_isolation_level(
+                                        eng.connect().connection)
         eq_(isolation_level, self._default_isolation_level())
 
     def test_reset_level(self):
         eng = testing_engine(options=dict())
         conn = eng.connect()
-        eq_(eng.dialect.get_isolation_level(conn.connection), self._default_isolation_level())
+        eq_(
+            eng.dialect.get_isolation_level(conn.connection),
+            self._default_isolation_level()
+        )
 
-        eng.dialect.set_isolation_level(conn.connection, self._non_default_isolation_level())
-        eq_(eng.dialect.get_isolation_level(conn.connection), self._non_default_isolation_level())
+        eng.dialect.set_isolation_level(
+                conn.connection, self._non_default_isolation_level()
+            )
+        eq_(
+            eng.dialect.get_isolation_level(conn.connection),
+            self._non_default_isolation_level()
+        )
 
         eng.dialect.reset_isolation_level(conn.connection)
-        eq_(eng.dialect.get_isolation_level(conn.connection), self._default_isolation_level())
+        eq_(
+            eng.dialect.get_isolation_level(conn.connection),
+            self._default_isolation_level()
+        )
 
         conn.close()
 
     def test_reset_level_with_setting(self):
-        eng = testing_engine(options=dict(isolation_level=self._non_default_isolation_level()))
+        eng = testing_engine(options=dict(
+                            isolation_level=
+                                self._non_default_isolation_level()))
         conn = eng.connect()
-        eq_(eng.dialect.get_isolation_level(conn.connection), self._non_default_isolation_level())
-
-        eng.dialect.set_isolation_level(conn.connection, self._default_isolation_level())
-        eq_(eng.dialect.get_isolation_level(conn.connection), self._default_isolation_level())
-
+        eq_(eng.dialect.get_isolation_level(conn.connection),
+            self._non_default_isolation_level())
+        eng.dialect.set_isolation_level(conn.connection,
+                self._default_isolation_level())
+        eq_(eng.dialect.get_isolation_level(conn.connection),
+            self._default_isolation_level())
         eng.dialect.reset_isolation_level(conn.connection)
-        eq_(eng.dialect.get_isolation_level(conn.connection), self._non_default_isolation_level())
-
+        eq_(eng.dialect.get_isolation_level(conn.connection),
+            self._non_default_isolation_level())
         conn.close()
 
     def test_invalid_level(self):
         eng = testing_engine(options=dict(isolation_level='FOO'))
         assert_raises_message(
-            exc.ArgumentError, 
+            exc.ArgumentError,
                 "Invalid value '%s' for isolation_level. "
-                "Valid isolation levels for %s are %s" % 
-                ("FOO", eng.dialect.name, ", ".join(eng.dialect._isolation_lookup)),
+                "Valid isolation levels for %s are %s" %
+                ("FOO", eng.dialect.name,
+                ", ".join(eng.dialect._isolation_lookup)),
             eng.connect)
 
     def test_per_connection(self):
         from sqlalchemy.pool import QueuePool
-        eng = testing_engine(options=dict(poolclass=QueuePool, pool_size=2, max_overflow=0))
+        eng = testing_engine(options=dict(
+                                poolclass=QueuePool,
+                                pool_size=2, max_overflow=0))
 
         c1 = eng.connect()
-        c1 = c1.execution_options(isolation_level=self._non_default_isolation_level())
-
+        c1 = c1.execution_options(
+                    isolation_level=self._non_default_isolation_level()
+                )
         c2 = eng.connect()
-        eq_(eng.dialect.get_isolation_level(c1.connection), self._non_default_isolation_level())
-        eq_(eng.dialect.get_isolation_level(c2.connection), self._default_isolation_level())
-
+        eq_(
+            eng.dialect.get_isolation_level(c1.connection),
+            self._non_default_isolation_level()
+        )
+        eq_(
+            eng.dialect.get_isolation_level(c2.connection),
+            self._default_isolation_level()
+        )
         c1.close()
         c2.close()
         c3 = eng.connect()
-        eq_(eng.dialect.get_isolation_level(c3.connection), self._default_isolation_level())
-
+        eq_(
+            eng.dialect.get_isolation_level(c3.connection),
+            self._default_isolation_level()
+        )
         c4 = eng.connect()
-        eq_(eng.dialect.get_isolation_level(c4.connection), self._default_isolation_level())
+        eq_(
+            eng.dialect.get_isolation_level(c4.connection),
+            self._default_isolation_level()
+        )
 
         c3.close()
         c4.close()
@@ -1234,17 +1404,20 @@ class IsolationLevelTest(fixtures.TestBase):
             r"on Connection.execution_options\(\), or "
             r"per-engine using the isolation_level "
             r"argument to create_engine\(\).",
-            select([1]).execution_options, isolation_level=self._non_default_isolation_level()
+            select([1]).execution_options,
+                    isolation_level=self._non_default_isolation_level()
         )
 
 
-    def test_per_engine_bzzt(self):
-        assert_raises_message(
-            exc.ArgumentError,
-            r"'isolation_level' execution option may "
-            r"only be specified on Connection.execution_options\(\). "
-            r"To set engine-wide isolation level, "
-            r"use the isolation_level argument to create_engine\(\).",
-            create_engine,
-            testing.db.url, execution_options={'isolation_level':self._non_default_isolation_level}
+    def test_per_engine(self):
+        # new in 0.9
+        eng = create_engine(testing.db.url,
+                            execution_options={
+                                'isolation_level':
+                                    self._non_default_isolation_level()}
+                        )
+        conn = eng.connect()
+        eq_(
+            eng.dialect.get_isolation_level(conn.connection),
+            self._non_default_isolation_level()
         )
