@@ -143,6 +143,26 @@ class DefaultRequirements(SuiteRequirements):
                 "not supported by database/driver"
             )
 
+    @property
+    def binary_literals(self):
+        """target backend supports simple binary literals, e.g. an
+        expression like::
+
+            SELECT CAST('foo' AS BINARY)
+
+        Where ``BINARY`` is the type emitted from :class:`.LargeBinary`,
+        e.g. it could be ``BLOB`` or similar.
+
+        Basically fails on Oracle.
+
+        """
+        # adding mssql here since it doesn't support comparisons either,
+        # have observed generally bad behavior with binary / mssql.
+
+        return skip_if(["oracle", "mssql"],
+                "not supported by database/driver"
+            )
+
     @property
     def independent_cursors(self):
         """Target must support simultaneous, independent database cursors
@@ -499,6 +519,7 @@ class DefaultRequirements(SuiteRequirements):
         return skip_if(['mssql', 'mysql', 'firebird', '+zxjdbc',
                     'oracle', 'sybase'])
 
+
     @property
     def precision_numerics_general(self):
         """target backend has general support for moderately high-precision
